#!/bin/python

import sys
import subprocess
import syslog

BIN_PING="/bin/ping"

def log(msg):
    syslog.syslog(syslog.LOG_ERR, msg)


def CmdExecutor(cmd_arg):
    try:
        retcode = subprocess.call(cmd_arg, stdout=subprocess.PIPE)

        if retcode < 0:
            log("{0} was terminated by signal: {1}".format(cmd_arg[0], -retcode))
            return False
        elif retcode != 0 and retcode != 10:
            log("{0} was returned: {1}".format(cmd_arg[0], retcode))
            return False
        else:
            return True
    except OSError as e:
        log("{0} execution failed: {1}".format(cmd_arg[0], e))
        return False


def ping_back(ip):
    '''
    ping 10.17.40.10 -c1 -W2
    '''

    cmd = [BIN_PING, ip, "-c1", "-W2"]
    return CmdExecutor(cmd)


def find_suitable_interface(ip_list):
    for ip in ip_list:
        if ping_back(ip):
            return ip


if __name__ == '__main__':
    ip_arg = sys.argv[1]
    if not ip_arg:
        exit(-1)

    ip_list = []
    for entry in ip_arg.split(","):
        ip_list.append(entry)

    ip = find_suitable_interface(ip_list)
    if ip:
        print(ip)
        exit(0)
    else:
        exit(-1)
