#!/bin/python
# Copyright (c) 2018 Synology Inc. All rights reserved.
import subprocess
import json
import ctypes
import syslog
import socket
import os

BIN_SQLITE3="/bin/sqlite3"
BIN_SYNO_GKV="/bin/get_key_value"
BIN_SYNO_GSKV="/usr/syno/bin/get_section_key_value"
BIN_SYNO_WEBAPI="/usr/syno/bin/synowebapi"


def ModelGet():
    cmd = "/bin/get_key_value /etc.defaults/synoinfo.conf unique"
    try:
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        return output.rstrip()
    except:
        return None


def VersionGet():
    cmd = "/bin/get_key_value /etc.defaults/VERSION productversion"
    try:
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        return output.rstrip()
    except:
        return None


def BuildNumberGet():
    cmd = "/bin/get_key_value /etc.defaults/VERSION buildnumber"
    try:
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        return output.rstrip()
    except:
        return None


def AccountGet():
    cmd = "/bin/get_key_value /etc/myds.conf mydsID"
    try:
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        return output.rstrip()
    except:
        return None


def VMMInstalledInfoGet(pkg_info):
    cmd = '/usr/syno/bin/synowebapi --exec api=SYNO.Core.Package method=get id=Virtualization additional=["installed_info"]'
    try:
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        output = json.loads(output.rstrip())
        path = output["data"]["additional"]["installed_info"]["path"]
        pkg_info["path"] = path[0:path.index('@appstore')]
    except:
        return


def PackageInfoArchive(packages):
    pkg_info_dir  = "/syno_migrate_pkginfo/"
    pkg_info_path = "/var/packages/{}/INFO"

    if os.path.exists(pkg_info_dir):
        cmd = ["rm", "-rf", pkg_info_dir]
        subprocess.call(cmd, stdout=subprocess.PIPE)
    os.mkdir(pkg_info_dir)

    for pkg in packages:
        # update VMM installed path
        if 'Virtualization' == pkg['id']:
            VMMInstalledInfoGet(pkg)

        # mkdir /syno_migrate_pkginfo/[pkg_id]/
        os.mkdir(pkg_info_dir+pkg['id'])

        # read /var/packages/[pkg_id]/INFO and write to
        # /syno_migrate_pkginfo/[pkg_id]/INFO excluding
        # pkg icon, localized description and displayname
        try:
            with open(pkg_info_dir+pkg['id']+"/INFO", "w") as w_file:
                with open(pkg_info_path.format(pkg['id']), "r") as r_file:
                    for line in r_file:
                        line = line.strip()
                        if line.startswith('package_icon') or \
                           line.startswith('description_') or \
                           line.startswith('displayname_'):
                            continue
                        w_file.write(line+'\n')
        except:
            continue

    cmd = ["tar", "-cf", pkg_info_dir+"syno_migrate_pkginfo.tar", pkg_info_dir]
    ret = subprocess.call(cmd, stdout=subprocess.PIPE)
    if 0 != ret:
        syslog.syslog("{} was returned: {}".format(cmd[0], ret))


def PackageInfoGet():
    cmd = "/usr/syno/bin/synowebapi --exec api=SYNO.Core.Package method=list version=1"
    pkg_info_path = "/var/packages/{}/INFO"
    try:
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        output = json.loads(output.rstrip())
        packages = output["data"]["packages"]
    except:
        return None

    PackageInfoArchive(packages)

    return packages


def StorageInfoGet():
    try:
        cmd = "/usr/syno/bin/synowebapi --exec api=SYNO.Storage.CGI.Storage method=load_info"
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        storage = json.loads(output.rstrip())

        return {
            "volumes": VolumeInfoGet(storage["data"]["volumes"]),
            "pools": storage["data"]["storagePools"],
            "blockLuns": BlockLUNInfoGet(storage["data"]["iscsiLuns"]),
            "ssdCaches": storage["data"]["ssdCaches"] if "ssdCaches" in storage["data"] else []
        }
    except:
        return None


def VolumeInfoGet(volumes):
    class mntent(ctypes.Structure):
        _fields_ = [
            ("mnt_fsname", ctypes.c_char_p),
            ("mnt_dir", ctypes.c_char_p),
            ("mnt_type", ctypes.c_char_p),
            ("mnt_opts", ctypes.c_char_p),
            ("mnt_freq", ctypes.c_int),
            ("mnt_nopass", ctypes.c_int)
        ]

    libc = ctypes.CDLL("libc.so.6")
    libc.getmntent.restype = ctypes.POINTER(mntent)
    pProc = libc.setmntent("/etc/fstab", "r")
    mntInfo = []

    while True:
        ret = libc.getmntent(pProc)
        if False == bool(ret): break

        for vol in volumes:
            if vol["vol_path"] == ret.contents.mnt_dir:
                mntInfo.append({
                    "refer_id": ret.contents.mnt_dir,
                    "local_disk": ret.contents.mnt_fsname,
                    "remote_disk": ret.contents.mnt_fsname,
                    "mnt_opt": ret.contents.mnt_opts,
                    "volume_id": vol["id"],
                    "fs_type": vol["fs_type"],
                    "size": vol["size"]["total_device"],
                    "atime_opt": vol.get("atime_opt", "relatime"),
                    "pool_path": vol["pool_path"],
                    "device_type": vol["device_type"],
                    "status": vol["status"],
                    "vol_attribute": vol.get("vol_attribute", "generic")
                })
                break

    libc.endmntent(pProc)
    return mntInfo


def BlockLUNInfoGet(luns):
    blockLun = []

    for lun in luns:
        if "block" == lun["iscsi_lun"]["device_type"]:
            blockLun.append({
                "refer_id": lun["iscsi_lun"]["name"],
                "rootpath": lun["iscsi_lun"]["rootpath"],
                "size": lun["iscsi_lun"]["size"],
                "status": lun["status"]
            })

    return blockLun



def ShareInfoGet():
    cmd = '/usr/syno/bin/synowebapi --exec api=SYNO.Core.Share method=list additional=["encryption"]'
    try:
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        shares = json.loads(output.rstrip())
        return shares["data"]["shares"]
    except:
        return None


def LunInfoGet():
    cmd = "/usr/syno/bin/synowebapi --exec api=SYNO.Core.ISCSI.LUN method=list"
    try:
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        lun = json.loads(output.rstrip())
        return lun["data"]["luns"]
    except:
        return None


def isSupportBtrfs():
    cmd = "/bin/get_key_value /etc.defaults/synoinfo.conf support_btrfs"
    try:
        output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
        return True if 'yes' == output.rstrip() else False
    except:
        return None


def IsDrbdSupported():
    return os.path.exists("/lib/modules/drbd.ko") and os.path.exists("/usr/bin/drbdadm")


def IsMailPlusAAMode():
    '''
    /var/packages/MailPlus-Server/target/bin/syno_mailserver_backend --getPeersNum
    0: error
    1: single mode
    2: AA
    '''

    syno_mailserver_backend = "/var/packages/MailPlus-Server/target/bin/syno_mailserver_backend"

    if not os.path.exists(syno_mailserver_backend):
        return False

    cmd = "{} --getPeersNum".format(syno_mailserver_backend)
    output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
    if "2" == output.rstrip():
        syslog.syslog(syslog.LOG_ERR, "MailPlus AA mode detected")
        return True

    return False


def isXA():
    return ("broadwellntbap" in ModelGet())


def runHA():
    cmd = "/bin/get_key_value /etc/synoinfo.conf runha"
    output, err = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE).communicate()
    if 'yes' == output.rstrip():
        return True

    return False


def DisableListGet(packages, storage):
    disable = []
    try:
        #HA
        if runHA() and not isXA():
            disable.append('HA')

        #Flash Cache
        if storage['ssdCaches'] and 0 < len(storage['ssdCaches']):
            disable.append('ssdCaches')

        #Time Backup
        for pkg in packages:
            if 'TimeBackup' == pkg['id']:
                disable.append('TimeBackup')
                break

        #MailPlus AA mode
        if IsMailPlusAAMode():
            disable.append('MailPlusAA')

        return disable
    except:
        return None

SZF_MIGRATION_SOURCE_CONFIG = "/usr/syno/etc/synomigration/info.cfg"
SZF_MIGRATION_TARGET_CONFIG = "/usr/syno/etc/packages/MigrationAssistant/info.cfg"
SZF_MIGRATION_TARGET_PROGRESS = "/usr/syno/etc/packages/MigrationAssistant/progress"
SZF_MIGRATION_REMOTE_ARCHIVE="/.syno_migration_archive.tar"

def MigrationSourceStatusGet():
    mgrt_cfg = open(SZF_MIGRATION_SOURCE_CONFIG, mode='r')
    if not mgrt_cfg:
        syslog.syslog(syslog.LOG_ERR, "fail to open migration config file: {}".format(SZF_MIGRATION_SOURCE_CONFIG))
        return False

    mgrt_json = json.load(mgrt_cfg)
    if not mgrt_json:
        syslog.syslog(syslog.LOG_ERR, "fail to load migration json {}".format(SZF_MIGRATION_SOURCE_CONFIG))
        return False

    if 1 == mgrt_json["version"]:
        return False

    return True


def MigrationTargetStatusGet():
    if not os.path.exists(SZF_MIGRATION_TARGET_PROGRESS):
        return False

    mgrt_progress = open(SZF_MIGRATION_TARGET_PROGRESS, mode='r')
    if not mgrt_progress:
        syslog.syslog(syslog.LOG_ERR, "fail to open migration progress file: {}".format(SZF_MIGRATION_TARGET_PROGRESS))
        return False

    mgrt_json = json.load(mgrt_progress)
    if not mgrt_json:
        syslog.syslog(syslog.LOG_ERR, "fail to load migration progress json {}".format(SZF_MIGRATION_TARGET_PROGRESS))
        return False

    # task is done
    if "reboot" == mgrt_json["main_stage"] and not os.path.exists(SZF_MIGRATION_REMOTE_ARCHIVE):
        return False

    # task is running
    if mgrt_json["status"] in ["running", "pausing", "paused", "cancelling"]:
        return True

    return False


def MigrationStatusGet():
    if os.path.exists(SZF_MIGRATION_SOURCE_CONFIG):
        return MigrationSourceStatusGet()

    if os.path.exists(SZF_MIGRATION_TARGET_CONFIG):
        return MigrationTargetStatusGet()

    return False


def FreeSizeGet():
    disk = os.statvfs('/')
    return disk.f_bfree * disk.f_bsize


def MemorySizeGet():

    libsynosdk = ctypes.cdll.LoadLibrary("/lib/libsynosdk.so.6")
    if not libsynosdk:
        log("fail to load library: {}".format("/lib/libsynosdk.so.6"))
        return 0

    libsynosdk.SLIBSystemPhysMemSizeGet.restype = ctypes.c_uint

    try:
        mem_size = libsynosdk.SLIBSystemPhysMemSizeGet()
        if 0 >= mem_size:
            syslog.syslog("fail to get memory size")
        return mem_size
    except Exception:
        syslog.syslog("fail to invoke SLIBSystemPhysMemSizeGet")
        return 0


def CpuBenchmarkGet():
    '''
    cat /proc/cpuinfo | grep -i bogomips
    '''
    try:
        unique = ModelGet()

        # hard-coding for alpine
        if 'alpine' in unique:
            return 13549.56

        bogomips_sum = 0
        with open("/proc/cpuinfo", "r") as file:
            for line in file:
                if 0 <= line.lower().find('bogomips'):
                    bogomips = line[line.find(':')+1:].strip()
                    bogomips_sum += float(bogomips)

        # hard-coding for cedarview
        if 'cedarview' in unique:
            bogomips_sum /= 2.0

        return bogomips_sum
    except:
        return -1


def InterfaceMTUGet(interface):
    '''
    get MTU by reading /sys/class/net/eth1/mtu
    '''

    mtu = 1500

    mtu_path = "/sys/class/net/{}/mtu".format(interface)
    if os.path.exists(mtu_path):
        f = open(mtu_path, "r")
        mtu = f.read().splitlines()[0]

    return mtu


def InterfaceIPGet(interface, ip_type = "inet "):
    '''
    ipv4 = os.popen('ip addr show eth0').read().split("inet ")[1].split("/")[0]
    ipv6 = os.popen('ip addr show eth0').read().split("inet6 ")[1].split("/")[0]
    '''

    ip = ""

    out = os.popen('ip addr show {}'.format(interface)).read().split(ip_type)
    if len(out) > 1:
        ip = out[1].split("/")[0]

    return ip


def InterfacesGet():
    ifaces = []

    for i in os.listdir('/sys/class/net/'):
        iface = {}
        iface["interface"] = i
        iface["ip_v4"] = InterfaceIPGet(i, "inet ")
        iface["ip_v6"] = InterfaceIPGet(i, "inet6 ")
        iface["mtu"] = InterfaceMTUGet(i)
        ifaces.append(iface)

    return ifaces


def LunBkpFullPathGet(lun_bkp_dst, share_list):
    share_post = lun_bkp_dst.find("/")
    if -1 == share_post:
        return ""

    share_name = lun_bkp_dst[:share_post]
    for s in share_list:
        if s["name"] == share_name:
            return s["vol_path"] + "/" + lun_bkp_dst + "/.lunbkp_meta"

    return ""


def LunBkpDstGet(share_list):
    lun_bkp_dst_list = []
    lun_bkp_confg = "/usr/syno/etc/lunbkp/lunbkptask.conf"

    if not os.path.exists(lun_bkp_confg):
        return lun_bkp_dst_list

    for line in open(lun_bkp_confg):
        i = line.find("full_path=")
        if -1 != i:
            lun_bkp_dst = line[i+10:].rstrip()
            full_path = LunBkpFullPathGet(lun_bkp_dst, share_list)
            lun_bkp_dst_list.append(full_path)

    return lun_bkp_dst_list


# Servcie status utilities
def IsDomainClientEnabled():
    cmd = [BIN_SYNO_GKV, "/etc/synoinfo.conf", "domainjoined"]
    try:
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        return True if 'yes' == output.rstrip() else False
    except:
        return False


def IsSSOClientEnabled():
    cmd = [BIN_SYNO_GKV, "/etc/synoinfo.conf", "sso_support"]
    try:
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        return True if 'yes' == output.rstrip() else False
    except:
        return False


def IsSharedFolderSyncEnabled():
    cmd = [BIN_SQLITE3, "/usr/syno/etc/S2S/config.sqlite", "select * from task_table"]
    try:
        s2s_task = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        return True if s2s_task.stdout.readline() else False
    except:
        return False


def IsDHCPServerEnabled():
    cmd = [BIN_SYNO_GKV, "/etc/dhcpd/dhcpd.info", "enable"]
    try:
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        return True if 'yes' == output.rstrip() else False
    except:
        return False


def IsDDSMEnabled():
    '''
    synowebapi --exec api=SYNO.Docker.Container method=list version=1 limit=-1 offset=0 type=ddsm | jq .data.total

    output:
        "containers" : [],
        "limit" : 0,
        "offset" : 0,
        "total" : 0
    '''
    try:
        cmd = [BIN_SYNO_WEBAPI, "--exec", "api=SYNO.Docker.Container", "method=list", "version=1",
                "list=-1", "offset=0", "type=ddsm"]
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        container = json.loads(output.rstrip())

        return (0 < container["data"]["total"])
    except:
        return False


# License status utilities
def HasSurveillanceStationLicense():
    '''
    synowebapi --exec api=SYNO.SurveillanceStation.License method=Load version=1 | jq .data.license_num
    '''
    try:
        cmd = [BIN_SYNO_WEBAPI, "--exec", "api=SYNO.SurveillanceStation.License", "method=Load", "version=1"]
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        license = json.loads(output.rstrip())

        return (0 < license["data"]["license_num"])
    except:
        return False


def HasPrestoFileServerLicense():
    '''
    synowebapi --exec api=SYNO.PrestoServer.License method=list version=1 | jq .data.license_status

    Output:
        no_license:     0
        have_license:   1  <= license purchase
        trial:          2  <= trial license
        trial_expired:  3
        limited_trial:  4  <= trial license
    '''
    try:
        cmd = [BIN_SYNO_WEBAPI, "--exec", "api=SYNO.PrestoServer.License", "method=list", "version=1"]
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        license = json.loads(output.rstrip())

        return (0 != license["data"]["license_status"])
    except:
        return False


def HasMcAfeeLicense():
    '''
    synowebapi --exec api=SYNO.AntiVirus.General method=get_sys_info | jq .data.licenseType

    Output:
        "licenceType" : "Unknown"
    '''
    try:
        cmd = [BIN_SYNO_WEBAPI, "--exec", "api=SYNO.AntiVirus.General", "method=get_sys_info", "version=1"]
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        license = json.loads(output.rstrip())

        return ("Unknown" != license["data"]["licenseType"])
    except:
        return False


def HasVMMLicense():
    '''
    synowebapi --exec api=SYNO.Virtualization.License.Pro method=list | jq .data.licenses
    '''
    try:
        cmd = [BIN_SYNO_WEBAPI, "--exec", "api=SYNO.Virtualization.License.Pro", "method=list", "version=1"]
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        license = json.loads(output.rstrip())

        return (0 < len(license["data"]["licenses"]))
    except:
        return False


def HasVDSMLicense():
    '''
    synowebapi --exec api=SYNO.Virtualization.License.VDSM method=list | jq .data.licenses
    '''
    try:
        cmd = [BIN_SYNO_WEBAPI, "--exec", "api=SYNO.Virtualization.License.VDSM", "method=list", "version=1"]
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        license = json.loads(output.rstrip())

        return (0 < len(license["data"]["licenses"]))
    except:
        return False


def HasDDSMLicense():
    '''
    synowebapi --exec api=SYNO.Docker.License method=list_by_device verison=1 appType=ddsm | jq .data.licenses
    '''
    try:
        cmd = [BIN_SYNO_WEBAPI, "--exec", "api=SYNO.Docker.License", "method=list_by_device", "version=1", "appType=ddsm"]
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        license = json.loads(output.rstrip())

        return (0 < len(license["data"]["licenses"]))
    except:
        return False


def IsNotifyMailEnable():
    cmd = [BIN_SYNO_GKV, "/usr/syno/etc/synosmtp.config", "eventsmtp"]
    try:
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        return True if output.rstrip() else False
    except:
        return False


def IsNotifySmsEnable():
    cmd = [BIN_SYNO_GSKV, "/usr/syno/etc/synosms.conf", "synosms", "sms_enable"]
    try:
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        return True if 'yes' == output.rstrip() else False
    except:
        return False


def IsNotifyPushEnable():
    cmd = [BIN_SYNO_GKV, "/etc/synoinfo.conf", "pushservice_mobile_enabled"]
    try:
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        return True if 'yes' == output.rstrip() else False
    except:
        return False


def RecommendLicenseGet(packages):
    subcommand = {
        # refer_id              package shortcut        license checker
        'SurveillanceStation':  ["SurveillanceStation", HasSurveillanceStationLicense],
        'PrestoServer':         ["PrestoServer",        HasPrestoFileServerLicense],
        'Virtualization':       ["Virtualization",      HasVMMLicense],
        'AntiVirus-McAfee':     ["AntiVirus-McAfee",    HasMcAfeeLicense],
        'VDSM':                 ["Virtualization",      HasVDSMLicense],
        'DDSM':                 ["Docker",              HasDDSMLicense]
    }

    license_list = list(filter(lambda s: subcommand[s][0] and subcommand[s][0] in packages and subcommand[s][1](), subcommand))

    return license_list


def RecommendSerivceGet():
    subcommand = {
        # refer_id            service checker
        'Domain Client':      IsDomainClientEnabled,
        'SSO Client':         IsSSOClientEnabled,
        'Shared Folder Sync': IsSharedFolderSyncEnabled,
        'DHCP Server':        IsDHCPServerEnabled
    }

    service_list = list(filter(lambda s: subcommand[s](), subcommand))
    return service_list


def RecommendPackageGet(packages):
    full_package_list = [
        "ActiveBackup", "CMS", "MailPlus-Server", "ActiveDirectoryServer", "Virtualization",
        "SnapshotReplication", "VPNCenter", "CloudSync", "HyperBackup", "GlacierBackup", "SynologyDrive"]

    package_list = list(filter(lambda p: p in full_package_list, packages))

    # special case: DDSM
    if "Docker" in packages and IsDDSMEnabled():
        package_list.append("Docker")

    return package_list


def NotificationGet():
    subcommand = {
        # refer_id  service checker
        'mail':     IsNotifyMailEnable,
        'sms':      IsNotifySmsEnable,
        'push':     IsNotifyPushEnable
    }
    service_list = list(filter(lambda s: subcommand[s](), subcommand))
    return service_list


def RecommendActionGet(account, licenses, services, packages):
    actions = []

    actions.append('StopService')

    if 0 != len(licenses):
        actions.append('MigrateLicense')

    if account:
        actions.append('SetupSynoAccount')

    if 0 != len(NotificationGet()):
        actions.append('SetupPushService')

    if 0 != len(services):
        actions.append('ReconfigureService')

    actions.append('RepairePackage')

    if 0 != len(packages):
        actions.append('ConfigurePackages')

    actions.append('RedirectExternalConn')

    return actions


def PlatformCheckListGet():
    feature_list = []

    if os.path.exists("/etc/power_sched.conf"):
        feature_list.append("power_schedule")

    return feature_list


def make_pkg_dict(packages):
    pkg_dict = {}

    if not isinstance(packages, list):
        return {}

    for pkg in packages:
        if pkg["id"] and pkg["name"]:
            pkg_dict[pkg["id"]] = pkg["name"]

    return pkg_dict


def make_pkg_name_list(pkg_id_list, pkg_dict):
    pkg_name_list = []

    for pkg_id in pkg_id_list:
        if pkg_id in pkg_dict:
            pkg_name_list.append(pkg_dict[pkg_id])
        else:
            pkg_name_list.append(pkg_id)

    return pkg_name_list


def main():
    dsinfo = {}
    packages = PackageInfoGet()
    storage = StorageInfoGet()
    account = AccountGet()

    dsinfo['model']    = ModelGet()
    dsinfo['version']  = VersionGet()
    dsinfo['buildnumber'] = BuildNumberGet()
    dsinfo['account']  = account
    dsinfo['storage']  = storage
    dsinfo['packages'] = packages
    dsinfo['shares']   = ShareInfoGet()
    dsinfo['hostname'] = socket.gethostname()
    dsinfo['support_drbd'] = IsDrbdSupported()
    dsinfo['support_btrfs'] = isSupportBtrfs()
    dsinfo['disable']  = DisableListGet(packages, storage)
    dsinfo['is_migrating'] = MigrationStatusGet()
    dsinfo['free_size'] = FreeSizeGet()
    dsinfo['mem_size'] = MemorySizeGet()
    dsinfo['cpu_benchmark'] = CpuBenchmarkGet()
    dsinfo['interfaces'] = InterfacesGet()
    dsinfo['lunbackup'] = LunBkpDstGet(dsinfo['shares'])

    pkg_dict = make_pkg_dict(packages)
    licenses = RecommendLicenseGet(pkg_dict)
    services = RecommendSerivceGet()
    pkgnames = RecommendPackageGet(pkg_dict)

    dsinfo['MigrateLicense'] = make_pkg_name_list(licenses, pkg_dict)
    dsinfo['ReconfigureService'] = services
    dsinfo['ConfigurePackages'] = make_pkg_name_list(pkgnames, pkg_dict)
    dsinfo['customized_recommendation'] = RecommendActionGet(account, licenses, services, pkgnames)
    dsinfo['platform_feature_check'] = PlatformCheckListGet()

    dsinfo_json = json.dumps(dsinfo, indent=4)
    print(dsinfo_json)


if __name__ == '__main__':
    main()
