#!/bin/python

import subprocess
import json
import sys

REMOTE_CONFIG="/usr/syno/etc/synomigration/info.cfg"


def GetSpaceInfo(spaces):
    spaceInfo = {};
    if type(spaces) != list:
        return spaceInfo;

    for space in spaces:
        if "vol_path" in space:
            spaceInfo[space["vol_path"]] = {
                "status": space["status"],
                "size": int(space["size"]["total_device"])
            }
        elif "iscsi_lun" in space and \
            "name" in space["iscsi_lun"] and \
            "block" == space["iscsi_lun"].get("device_type", ""):
            spaceInfo[space["iscsi_lun"]["name"]] = {
                "status": space["status"],
                "size": int(space["iscsi_lun"]["size"])
            }

    return spaceInfo


def SpaceInfoGet():
    cmd = ["/usr/syno/bin/synowebapi",
            "--exec",
            "api=SYNO.Storage.CGI.Storage",
            "method=load_info"]
    try:
        output, err = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
        storage = json.loads(output.rstrip())

        return dict(GetSpaceInfo(storage["data"]["volumes"]).items() + \
               GetSpaceInfo(storage["data"]["iscsiLuns"]).items())
    except:
        return {}


def CheckSpace(infoCfg, spaceInfo):
    if "migration_setting" not in infoCfg or \
        "volume_mapping" not in infoCfg["migration_setting"]:
        return -1

    spaces = infoCfg["migration_setting"]["volume_mapping"]
    for space in spaces:
        if space["mnt_point"] not in spaceInfo:
            print "The space [{0}] is missing".format(space["mnt_point"])
            return -1
        elif spaceInfo[space["mnt_point"]].get("status", "") not in ["normal", "background"]:
            print "The status of the current space [{0}] is [{1}].".format(space["mnt_point"], spaceInfo[space["mnt_point"]].get("status", ""))
            return -1
        elif spaceInfo[space["mnt_point"]].get("size", 0) != space["size"]:
            print "The size of the current space [{0}] is changed. ori: {1}, cur: {2}".format(space["mnt_point"], space["size"], spaceInfo[space["mnt_point"]].get("size", 0))
            return -1

    return 0


with open(REMOTE_CONFIG) as config:
    infoCfg = json.load(config)

sys.exit(CheckSpace(infoCfg, SpaceInfoGet()))
