#!/bin/python

import sdk
import subprocess

BIN_DRBDADM="/bin/drbdadm"
BIN_DRBDSETUP="/bin/drbdsetup"

# drbd config relative definition
DIR_SYNO_DRBD="/usr/syno/etc/synomigration/drbd.d/"
CFG_SYNO_DRBD=DIR_SYNO_DRBD+"drbd.conf"
OPT_SYNO_DRBD="--config-file="+CFG_SYNO_DRBD

def DrbdExecutor(cmd_arg):
    try:
        retcode = subprocess.call(cmd_arg, stdout=subprocess.PIPE)

        if retcode < 0:
            sdk.log("{0} was terminated by signal: {1}".format(cmd_arg[0], -retcode))
            return False
        elif retcode != 0 and retcode != 10:
            sdk.log("{0} was returned: {1}".format(cmd_arg[0], retcode))
            return False
        else:
            return True
    except OSError as e:
        sdk.log("{0} execution failed: {1}".format(cmd_arg[0], e))
        return False


#######################################################################
# setup relative command
#######################################################################

def DrbdCreateMD(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "create-md", resource_name]
    return DrbdExecutor(cmd_arg)


def DrbdNewResource(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "new-resource", resource_name]
    return DrbdExecutor(cmd_arg)


def DrbdShNewMinor(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "sh-new-minor", resource_name]
    return DrbdExecutor(cmd_arg)


def DrbdAttach(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "attach", resource_name]
    return DrbdExecutor(cmd_arg)


def DrbdPrimary(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "primary", resource_name, "--force"]
    return DrbdExecutor(cmd_arg)


#######################################################################
# tear down relative command
#######################################################################

def DrbdPauseSync(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "pause-sync", resource_name]
    return DrbdExecutor(cmd_arg)


def DrbdSecondary(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "secondary", resource_name]
    return DrbdExecutor(cmd_arg)


def DrbdDisconnect(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "disconnect", resource_name]
    return DrbdExecutor(cmd_arg)


def DrbdDown(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "down", resource_name]
    return DrbdExecutor(cmd_arg)


def DrbdDownExtra(resource_name):
    cmd_arg = [BIN_DRBDSETUP, "down", resource_name]
    return DrbdExecutor(cmd_arg)

#######################################################################
# data sync relative command
#######################################################################

def DrbdConnect(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "connect", resource_name]
    return DrbdExecutor(cmd_arg)


def DrbdResumeSync(resource_name):
    cmd_arg = [BIN_DRBDADM, OPT_SYNO_DRBD, "resume-sync", resource_name]
    return DrbdExecutor(cmd_arg)
