#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright (C) 2000-2019 Synology Inc. All rights reserved.
#
# Created by:   jamiec
# Created on:   2019-01-03

import os
import sys
import sdk
from shutil import copy2

FEA_POWER_OFF_CONF="MigrationAssistant-migration_poweroff.cfg"
FEA_VOL_DELETE_CONF="MigrationAssistant-migration_volume_delete.cfg"
FEA_VOL_EXPAND_CONF="MigrationAssistant-migration_volume_expand.cfg"
FEA_POOL_DELETE_CONF="MigrationAssistant-migration_pool_delete.cfg"
FEA_BOND_CREATE_CONF="MigrationAssistant-migration_bond_create.cfg"
FEA_BOND_DELETE_CONF="MigrationAssistant-migration_bond_delete.cfg"
FEA_OVS_CREATE_CONF="MigrationAssistant-migration_ovs_create.cfg"
FEA_OVS_DELETE_CONF="MigrationAssistant-migration_ovs_delete.cfg"
FEA_DISK_DEACTIVATE_CONF="MigrationAssistant-migration_disk_deactivate.cfg"

DIR_MIGRATION_ETC="/usr/syno/etc/synomigration/feasibility.d/"
FEA_POWER_OFF_SRC=DIR_MIGRATION_ETC+FEA_POWER_OFF_CONF
FEA_VOL_DELETE_SRC=DIR_MIGRATION_ETC+FEA_VOL_DELETE_CONF
FEA_VOL_EXPAND_SRC=DIR_MIGRATION_ETC+FEA_VOL_EXPAND_CONF
FEA_POOL_DELETE_SRC=DIR_MIGRATION_ETC+FEA_POOL_DELETE_CONF
FEA_BOND_CREATE_SRC=DIR_MIGRATION_ETC+FEA_BOND_CREATE_CONF
FEA_BOND_DELETE_SRC=DIR_MIGRATION_ETC+FEA_BOND_DELETE_CONF
FEA_OVS_CREATE_SRC=DIR_MIGRATION_ETC+FEA_OVS_CREATE_CONF
FEA_OVS_DELETE_SRC=DIR_MIGRATION_ETC+FEA_OVS_DELETE_CONF
FEA_DISK_DEACTIVATE_SRC=DIR_MIGRATION_ETC+FEA_DISK_DEACTIVATE_CONF

DIR_SYNO_FEACHECK="/usr/local/share/feasibilitycheck/"
FEA_POWER_OFF_DST=DIR_SYNO_FEACHECK+"poweroff.d/"
FEA_VOL_DELETE_DST=DIR_SYNO_FEACHECK+"volume.d/delete/"
FEA_VOL_EXPAND_DST=DIR_SYNO_FEACHECK+"volume.d/expand/"
FEA_POOL_DELETE_DST=DIR_SYNO_FEACHECK+"pool.d/delete/"
FEA_BOND_CREATE_DST=DIR_SYNO_FEACHECK+"net.d/bond_create/"
FEA_BOND_DELETE_DST=DIR_SYNO_FEACHECK+"net.d/bond_delete/"
FEA_OVS_CREATE_DST=DIR_SYNO_FEACHECK+"net.d/ovs_create/"
FEA_OVS_DELETE_DST=DIR_SYNO_FEACHECK+"net.d/ovs_delete/"
FEA_DISK_DEACTIVATE_DST=DIR_SYNO_FEACHECK+"disk.d/deactivate/"

def CreateFolderIfNotExisted(folder_path):
    if os.path.isdir(folder_path):
        return True

    if os.path.exists(folder_path):
        # existed but not a folder
        return False

    os.makedirs(folder_path)
    return True


def RegisterFeasibilityCheck(src, dst, conf):
    # check dst folder existence
    if not CreateFolderIfNotExisted(dst):
        sdk.log("fail to create folder: {}".format(dst))
        return False

    # check dst file existence
    if os.path.exists(dst+conf):
        os.unlink(dst+conf)

    # copy file
    try:
        copy2(src, dst+conf)
    except IOError, e:
        sdk.log("fail to copy file: {0} {1}".format(src, dst))
        return False


def UnlinkFile(filename):
    try:
        if os.path.exists(filename):
            os.unlink(filename)
    except OSError:
        sdk.log("fail to unlink file: {}".format(filename))
        return False

    return True


def RegisterAllFeasibilityCheck():
    RegisterFeasibilityCheck(FEA_POWER_OFF_SRC, FEA_POWER_OFF_DST, FEA_POWER_OFF_CONF)
    RegisterFeasibilityCheck(FEA_VOL_DELETE_SRC, FEA_VOL_DELETE_DST, FEA_VOL_DELETE_CONF)
    RegisterFeasibilityCheck(FEA_VOL_EXPAND_SRC, FEA_VOL_EXPAND_DST, FEA_VOL_EXPAND_CONF)
    RegisterFeasibilityCheck(FEA_POOL_DELETE_SRC, FEA_POOL_DELETE_DST, FEA_POOL_DELETE_CONF)
    RegisterFeasibilityCheck(FEA_BOND_CREATE_SRC, FEA_BOND_CREATE_DST, FEA_BOND_CREATE_CONF)
    RegisterFeasibilityCheck(FEA_BOND_DELETE_SRC, FEA_BOND_DELETE_DST, FEA_BOND_DELETE_CONF)
    RegisterFeasibilityCheck(FEA_OVS_CREATE_SRC, FEA_OVS_CREATE_DST, FEA_OVS_CREATE_CONF)
    RegisterFeasibilityCheck(FEA_OVS_DELETE_SRC, FEA_OVS_DELETE_DST, FEA_OVS_DELETE_CONF)
    RegisterFeasibilityCheck(FEA_DISK_DEACTIVATE_SRC, FEA_DISK_DEACTIVATE_DST, FEA_DISK_DEACTIVATE_CONF)
    return True


def UnregisterAllFeasibilityCheck():
    UnlinkFile(FEA_POWER_OFF_DST + FEA_POWER_OFF_CONF)
    UnlinkFile(FEA_VOL_DELETE_DST + FEA_VOL_DELETE_CONF)
    UnlinkFile(FEA_VOL_EXPAND_DST + FEA_VOL_EXPAND_CONF)
    UnlinkFile(FEA_POOL_DELETE_DST + FEA_POOL_DELETE_CONF)
    UnlinkFile(FEA_BOND_CREATE_DST + FEA_BOND_CREATE_CONF)
    UnlinkFile(FEA_BOND_DELETE_DST + FEA_BOND_DELETE_CONF)
    UnlinkFile(FEA_OVS_CREATE_DST + FEA_OVS_CREATE_CONF)
    UnlinkFile(FEA_OVS_DELETE_DST + FEA_OVS_DELETE_CONF)
    UnlinkFile(FEA_DISK_DEACTIVATE_DST + FEA_DISK_DEACTIVATE_CONF)
    return True


def register_feasibility_check():
    RegisterAllFeasibilityCheck()
    exit(0)


def unregister_feasibility_check():
    UnregisterAllFeasibilityCheck()
    exit(0)


if __name__ == '__main__':
    subcommand = {
        'register': register_feasibility_check,
        'unregister': unregister_feasibility_check
    }
    if len(sys.argv) > 1 and sys.argv[1] in subcommand:
        subcommand[sys.argv[1]]()
    else:
        sdk.log("invalid argument")
        exit(-1)

# vim:set shiftwidth=4 tabstop=4 expandtab:
