#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright (C) 2000-2018 Synology Inc. All rights reserved.
#
# Created by:   lukewu
# Created on:   2018-12-25

import sys
import os
import subprocess
import json
import sdk

MIGRATION_ASSISTANT_LOCK = '/run/synomigrationassistant_start.lock'

'''
lock/unlock migration assistant to stop stage.
'''


def usage(exit_code):
    print('usage: {0} <lock|unlock|check_lock>'.format(sys.argv[0]))
    exit(exit_code)


def lock():
    '''
    set the lock flag and stop the migration assistant
    '''
    package_check_stop_cmd = [
            '/usr/syno/bin/synowebapi', '--exec',
            'api=SYNO.Core.Package', 'method=feasibility_check',
            'type=stop_check', 'packages=["MigrationAssistant"]'
            ]
    package_stop_cmd = ['/usr/syno/bin/synopkg', 'stop', 'MigrationAssistant']
    try:
        with open(MIGRATION_ASSISTANT_LOCK, 'w'):
            pass

        # package feasibility checks exists since DSM6.1beta
        if int(sdk.SynoVersionGet()) >= sdk.DSM_61_START_VERSION:
            output, err = subprocess.Popen(
                    package_check_stop_cmd, stdout=subprocess.PIPE).communicate()
            resp = json.loads(output.strip())
            if not resp['success']:
                raise RuntimeError('package feasibility check stop the process '
                                   'to stop migration assistant')

        if not sdk.CmdExecutor(package_stop_cmd):
            raise RuntimeError('it\'s failed to stop MigrationAssistant')
    except Exception as e:
        sdk.log("failed to lock with {0} because {1}".format(
            MIGRATION_ASSISTANT_LOCK, e))
        if os.path.isfile(MIGRATION_ASSISTANT_LOCK):
            os.unlink(MIGRATION_ASSISTANT_LOCK)
        exit(-1)


def unlock():
    '''
    set the unlock flag
    '''
    try:
        if os.path.isfile(MIGRATION_ASSISTANT_LOCK):
            os.unlink(MIGRATION_ASSISTANT_LOCK)
    except Exception as e:
        sdk.log("failed to unlock with {0} because {1}".format(
            MIGRATION_ASSISTANT_LOCK, e))
        exit(-1)


def check_lock():
    if os.path.isfile(MIGRATION_ASSISTANT_LOCK):
        print("locked")
        exit(0)
    print("unlocked")
    exit(1)


if __name__ == '__main__':
    subcommand = {
        'lock': lock,
        'unlock': unlock,
        'check_lock': check_lock
    }
    if len(sys.argv) > 1 and sys.argv[1] in subcommand:
        subcommand[sys.argv[1]]()
    else:
        usage(-1)

# vim:set shiftwidth=4 tabstop=4 expandtab:
