#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright (C) 2000-2019 Synology Inc. All rights reserved.
#
# Created by:   jamiec
# Created on:   2019-11-28

import sys
import os
import sdk

TASK_NAME="Migration Assistant Task Cleaner"
TASK_SCRIPT="/usr/syno/etc/synomigration/scripts/python/space/resource_cleaner.py"

def task_create():
    '''
    synowebapi --exec api=SYNO.Core.TaskScheduler method=create version=2
        name=$NAME owner=root enable=false type=script
        extra={"script": $SCRIPT}
        schedule={"date_type":0,"week_day":"0,1,2,3,4,5,6","hour":0,"minute":0,"repeat_hour":0,"last_work_hour":0}
    '''

    # delete task first
    task_delete()

    kwargs = {
        'name': TASK_NAME,
        'enable': False,
        'owner': 'root',
        'type': 'script',
        'extra': {
             "script": TASK_SCRIPT
        },
        'schedule': {
            'date_type': 0,
            'week_day': "0",
            'hour': 0,
            'minute': 0,
            'repeat_hour': 0,
            'last_work_hour': 0
        }
    }
    j_data = sdk.execWebAPI('SYNO.Core.TaskScheduler', 1, 'create', **kwargs)
    if not j_data or not j_data["success"]:
        return False

    return True


def task_list():
    '''
    synowebapi --exec api=SYNO.Core.TaskScheduler method=list version=1
    '''

    j_data = sdk.execWebAPI('SYNO.Core.TaskScheduler', 1, 'list')
    if not j_data or not j_data["success"]:
        return False

    return j_data["data"]["tasks"]


def task_id_get():
    id_list = []

    tasks = task_list()
    for t in tasks:
        if TASK_NAME == t['name']:
            id_list.append(t['id'])

    return id_list


def task_delete():
    '''
    synowebapi --exec api=SYNO.Core.TaskScheduler method=delete version=1 task=[$ID]
    '''

    # get task id
    id_list = task_id_get()
    if (0 == len(id_list)):
        return True

    # delete task by id
    j_data = sdk.execWebAPI('SYNO.Core.TaskScheduler', 1, 'delete', task=id_list)
    if not j_data or not j_data["success"]:
        return False

    return True


if __name__ == '__main__':
    subcommand = {
        'create': task_create,
        'delete': task_delete,
        'list': task_list
    }
    if len(sys.argv) > 1 and sys.argv[1] in subcommand:
        ret = subcommand[sys.argv[1]]()
        if (ret):
            exit(0)
        else:
            exit(-1)
    else:
        sdk.log("invalid argument")
        exit(-1)

# vim:set shiftwidth=4 tabstop=4 expandtab:
