#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright (C) 2000-2019 Synology Inc. All rights reserved.
#
# Created by:   jamiec
# Created on:   2019-01-03

import sys
import sdk
import time


def IsShellRefreshed(user_name):
    for line in open("/etc/passwd"):
        if user_name in line and "/bin/sh" in line:
            return True
    return False


def WaitShellRefresh(user_name, count):
    '''
    Wait till remote shell of specified user is updated:
    MigrationAssistant:x:1034:100::/var/services/homes/MigrationAssistant:/sbin/nologin
    MigrationAssistant:x:1034:100::/var/services/homes/MigrationAssistant:/bin/sh

    '''

    for i in range(count):
        if IsShellRefreshed(user_name):
            return True
        time.sleep(1)

    sdk.log("fail to wait package user {} shell refresh".format(user_name))
    return False

def PackageUserCreate(user_name, password):
    desc = 'User created for Migration Assistant'
    group = 'administrators'

    if sdk.IsUserExisted(user_name):
        sdk.log("package user is already existed")
        return True

    if not sdk.UserCreate(user_name, password, desc):
        sdk.log("fail to create migration package user: {}".format(user_name))
        return False

    if not sdk.UserGroupJoin(user_name, group):
        sdk.log("fail to join group")
        return False

    # wait sdk write modification
    sdk.FileSync()
    sdk.FileSync()
    sdk.FileSync()

    # wait till shell refresh
    if not WaitShellRefresh(user_name, 20):
        sdk.log("fail to wait shell refresh")
        return False

    if not sdk.UserHomeDisable(user_name):
        sdk.log("fail to disable home")
        return False

    return True


def PackageUserDelete(user_name):
    if not sdk.IsUserExisted(user_name):
        return True

    if not sdk.UserDelete(user_name):
        sdk.log("fail to delete migration package user: {}".format(user_name))
        return False

    return True


def user_create():
    user = sys.argv[2]
    pswd = sys.argv[3]

    if not PackageUserCreate(user, pswd):
        exit(-1)
    exit(0)


def user_delete():
    user = sys.argv[2]

    if not PackageUserDelete(user):
        exit(-1)
    exit(0)


if __name__ == '__main__':
    subcommand = {
        'create': user_create,
        'delete': user_delete
    }
    if len(sys.argv) > 1 and sys.argv[1] in subcommand:
        subcommand[sys.argv[1]]()
    else:
        sdk.log("invalid argument")
        exit(-1)

# vim:set shiftwidth=4 tabstop=4 expandtab:
